#!/bin/sh

############################################################
# 函数定义

# 暂停操作
function pause() {
    read -n1 -p "Press any key to continue..." ;
}

# 强制退出
function force_exit()
{
    # 删除安装临时文件
    rm -rf "/tmp/._umail_install_tmp" ;
    rm -rf "/tmp/._umail_flag_confbuilder" ;

    # 输出信息
    echo "Info: The installation process is interrupted!" ;
    echo ;
    exit 1 ;
}

# 取消安装
function cancel_install() {
    clear ;
    echo ;
    echo "Info: installation was canceled by the user!" ;
    force_exit ;
}

# 检测指定的端口是否已被使用
function check_port_is_use() {
    netstat -nat | grep "LISTEN" | awk '{print $4}' | egrep ":$1$" >/dev/null 2>&1 ;
    return $? ;
}

# 检测指定的 RPM 包是否已被安装
function check_rpm_pack() {
    if [ "$1" == "" ]; then
        echo "Error: check rpm package param error!" ;
        exit 1 ;
    fi
    rpm -q "$1" >/dev/null 2>&1 ;
    return $? ;
}

# 安装指定的 RPM 包
# install_rpm_pack <软件包名称> <软件包地址>
function install_rpm_pack()
 {
    # 参数检测
    if [ "$1" == "" ] || [ "$2" == "" ] ; then
        echo "Error: install package param error ($1)" ;
        exit 1 ;
    fi
    local PACK_NAME="$1" ;
    local PACK_PATH="$2" ;

    # 检测指定的软件包是否已安装
    if check_rpm_pack "${PACK_NAME}" ;then
        return 0 ;
    fi

    # 检测指定的文件是否存在并唯一
    local FILE_LIST=`ls ${PACK_PATH} 2>/dev/null` ;
    local CHAR_COUNT=`echo "${FILE_LIST}" | wc -w` ;
    local LINE_COUNT=`echo "${FILE_LIST}" | wc -l` ;
    if [ ${CHAR_COUNT} -eq 0 ] ; then
        echo "Error: can't found the \"${PACK_NAME}\" package!" ;
        exit 1 ;
    fi
    if [ ${LINE_COUNT} -gt 1 ]; then
        echo "Error: the package \"${PACK_NAME}\" is not unique!" ;
        exit 1 ;
    fi

    # 安装 RPM 包
    echo "Install package:" `basename ${FILE_LIST}`;
    rpm -ivh ${FILE_LIST} ;
    if [ ! $? -eq 0 ]; then
        echo "Error: install rpm package failure!" ;
        exit 1;
    fi
}


