#!/bin/sh

# 设置路径
ROOT_PATH="$(cd `dirname $0`; pwd)";
echo $ROOT_PATH;
TEMP_PATH="/tmp/._umail_install_tmp" ;
RESOURCE_PATH="$ROOT_PATH/resources" ;
PACKAGE_PATH="$ROOT_PATH/packages" ;
DEPEND_PATH="$PACKAGE_PATH/dependent" ;

# 加载相关文件
. "$ROOT_PATH/func_common.sh" ;

############################################################
# 函数定义

             
############################################################
# 变量设置
UPGRADE_STEP=0                              # 更新步骤计数器

VAR_DOMAIN_NAME="" ;
VAR_MAILBOX_PATH="" ;
VAR_NGINX_PORT="" ;
VAR_APACHE_PORT="" ;
VAR_PGSQL_UMAIL_PASS="" ;
VAR_SUPER_ADMIN_PASS="" ;
VAR_DOMAIN_ADMIN_PASS="" ;
VAR_SYSTEM_ADMIN_PASS="" ;
# Yum repo 源设置
REPO_UPGRADE_URL='http://update.comingchina.com:8080/repo/umail_beta.repo'
#USE_REPO_LIST='base,updates,umail_stable,umail_beta'
USE_REPO_LIST='umail_beta'


if [ -x /sbin/runuser ]
then
    SU=runuser
else
    SU=su
fi

############################################################
# 函数定义
# 检测 yum 程序
check_yum_program()
{
    [ -e /usr/bin/yum ] && return 0
    echo "Error: Can't found yum command!"
    exit 2
}

# 输出更新步骤头
output_header()
{
let UPGRADE_STEP=UPGRADE_STEP+1
    echo
    echo "#"
    echo "# Step$UPGRADE_STEP: $1"
    echo "#"
    echo
}

# 升级 repo 源文件
upgrade_yum_repo()
{
    output_header "Upgrade yum repos"

    # 取得最新的 repo 文件
    wget -O /etc/yum.repos.d/umail.repo -o /dev/null ${REPO_UPGRADE_URL} >/dev/null 2>&1
    # 升级本地资源库
    yum --disablerepo=* --enablerepo=${USE_REPO_LIST} clean all
    /usr/local/u-mail/app/engine/bin/pip install requests
}


# 安装 升级 包
function install_package() {
    yum -y install perl-Mail-MboxParser p7zip
    for n in postgresql94-server dspam dspam-client dspam-pgsql spamfilter
    do
        local packname="umail_$n"
        if ! rpm -q ${packname} >/dev/null 2>&1; then
            local wait_install_pack="${wait_install_pack} ${packname}"
        fi
    done
    if [ ! "${wait_install_pack}" = "" ]; then
        echo "***** install packages: ${wait_install_pack} *****"
        yum -y --disablerepo=* --enablerepo=${USE_REPO_LIST} install ${wait_install_pack}
    fi
    yum -y --enablerepo=${USE_REPO_LIST} reinstall umail_spamfilter
}


# 从临时文件中读出设置参数
# 修改用户自定义配置文件
function modify_custom_setting()
{
    echo "### Modify custom setting ###";
    local CUSTOM_FILE_PATH="/usr/local/u-mail/config/custom.conf" ;
    if [[ `grep -c "pg_pass" ${CUSTOM_FILE_PATH}` -eq 0 ]]; then
    echo "
[pgsql]
pg_host   = 127.0.0.1
pg_port   = 5432
pg_user   = umail
pg_pass   = 123456
dbname_dspam = dspam" >> ${CUSTOM_FILE_PATH};

    # 如果已存在配置文件，则对其进行备份
    #if [ -e "${CUSTOM_FILE_PATH}" ]; then
    #    local DATETIME=$(date +%Y%m%d%H%M%S) ;
    #    mv "${CUSTOM_FILE_PATH}" "${CUSTOM_FILE_PATH}_${DATETIME}" ;
    #fi

    # 生成 MySQL 的 umail 用户密码
    VAR_PGSQL_UMAIL_PASS=$(cat /dev/urandom | head -1 | md5sum | head -c 15) ;

    # 修改配置文件
    #cp "/usr/local/u-mail/config/template/custom.conf" "${CUSTOM_FILE_PATH}" ;
    perl -p -i -e "s/123456/${VAR_PGSQL_UMAIL_PASS}/ if /^pg_pass\s*=/;" "${CUSTOM_FILE_PATH}" ;
else
    VAR_PGSQL_UMAIL_PASS=`awk '$1=="pg_pass" {print $3}' ${CUSTOM_FILE_PATH}`
fi
}

# 处理 Postgresql 服务
function process_pgsql_service()
{
    echo "### Initialize PgSQL services ###";

    # 创建 MySQL 的配置文件
    #if [ ! -d /usr/local/u-mail/config/mysql ]; then
    #    /usr/local/u-mail/app/sbin/confbuilder mysql >/dev/null 2>&1 ;
    #fi

    # 备份原数据
    #if [ -d /usr/local/u-mail/data/mysql/default ]; then
    #    local DATETIME=$(date +%Y%m%d%H%M%S) ;
    #    mv "/usr/local/u-mail/data/mysql/default" "/usr/local/u-mail/data/mysql/default_${DATETIME}" ;
    #fi

    # 初始　启动 PgSQL 服务
    /sbin/chkconfig umail_postgresql --level 345 on >/dev/null 2>&1 ;
    /sbin/service umail_postgresql initdb >/dev/null 2>&1;

    #修改postgres 配置文件
    perl -p -i -e "s/ident/md5/ if /^host\s*all\s*all\s*127.0.0.1/;" "/usr/local/u-mail/service/pgsql-9.4/data/data/pg_hba.conf";
    /sbin/service umail_postgresql start >/dev/null 2>&1;
    if [ $? -ne 0 ]; then
        echo "Error: start postgresql service failure" ;
        force_exit ;
    fi
}


# 处理 dspam 服务
function process_dspam_service()
{
    /usr/local/u-mail/app/engine/bin/python /usr/local/u-mail/app/repo/confbuilder.pyc dspam
    echo "### Initialize dspam services ###";

    # 初始　启动 dspam 服务
    /sbin/chkconfig umail_dspam --level 345 on >/dev/null 2>&1 ;

    /sbin/service umail_dspam start >/dev/null 2>&1;
    if [ $? -ne 0 ]; then
        echo "Error: start dspam service failure" ;
        force_exit ;
    fi
}


# 处理 PgSQL 中的 umail 数据库与用户
function process_pgsql_umail()
{
    echo "### Initialize U-Mail database ###";

    # 生成临时 SQL 文件
    rm -f "${RESOURCE_PATH}/umail_user-temp.sql" ;
    cp "${RESOURCE_PATH}/umail_pg_user.sql" "${RESOURCE_PATH}/umail_pg_user-temp.sql" ;


    # 修改临时 SQL 文件
    perl -p -i -e "s/pgsql_umail_password/${VAR_PGSQL_UMAIL_PASS}/;" "${RESOURCE_PATH}/umail_pg_user-temp.sql" ;


    # 将 SQL 文件导入至数据库
    local PGSQL_PATH="/usr/local/u-mail/service/pgsql-9.4/bin/psql";
    $SU -l umail_postgres -c "${PGSQL_PATH} -U umail_postgres -d postgres  < ${RESOURCE_PATH}/umail_pg_user-temp.sql" ;
    #$SU -l umail_postgres -c "${PGSQL_PATH} -U umail_postgres -d dspam -f /usr/share/dspam/sql-scripts/pgsql/pgsql_objects.sql" ;
    #$SU -l umail_postgres -c "${PGSQL_PATH} -U umail_postgres -d dspam -f /usr/share/dspam/sql-scripts/pgsql/virtual_users.sql" ;
    PGPASSWORD=${VAR_PGSQL_UMAIL_PASS} /usr/local/u-mail/service/pgsql-9.4/bin/psql -U umail -d dspam -h 127.0.0.1 -f /usr/share/dspam/sql-scripts/pgsql/pgsql_objects.sql ;
    PGPASSWORD=${VAR_PGSQL_UMAIL_PASS} /usr/local/u-mail/service/pgsql-9.4/bin/psql -U umail -d dspam -h 127.0.0.1 -f /usr/share/dspam/sql-scripts/pgsql/virtual_users.sql ;

    if [ $? -ne 0 ]; then
        echo "Error: import database failure!" ;
        force_exit ;
    fi
    # 删除临时 SQL 文件
    rm -f "${RESOURCE_PATH}/umail_user-temp.sql" ;
}


function main() {
    echo ;
    echo '#' ;
    echo '# Setting Packages' ;
    echo '#' ;
    echo ;

    # 检测 yum 程序
    check_yum_program

    upgrade_yum_repo

    # 修改用户自定义配置文件
    modify_custom_setting ;

    install_package ;

    # 处理 PGSQL 服务
    process_pgsql_service ;

    # 处理 PGSQL 中的 umail 数据库与用户
    process_pgsql_umail ;

    # 处理 dspam 服务
    process_dspam_service ;
}


############################################################
# 程序执行

main ;
exit 0 ;


